/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ...
 * @author     Jianfeng Gao ( jianfeng dot gao at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/control/common/mps/KeypointsMP.h>
#include <armarx/control/common/mps/JSMP.h>
#include <armarx/control/common/mps/MPPool.h>
#include <armarx/control/njoint_controller/task_space/KeypointsImpedanceController.h>
//#include <armarx/control/njoint_mp_controller/task_space/ControllerInterface.h>


namespace armarx::control::njoint_mp_controller::task_space
{
    using namespace armarx::control::njoint_controller::task_space;
    using namespace armarx::control::common;
    /**
    * @defgroup Library-NJointKeypointsImpedanceMPController NJointKeypointsImpedanceMPController
    * @ingroup Library-RobotUnit-NJointControllers
    * A description of the library NJointKeypointsImpedanceMPController.
    *
    * @class NJointKeypointsImpedanceMPController
    * @ingroup Library-NJointKeypointsImpedanceMPController
    * @brief Brief description of class NJointKeypointsImpedanceMPController.
    *
    * Detailed description of class NJointKeypointsImpedanceMPController.
    */
    class NJointKeypointsImpedanceMPController :
            virtual public NJointKeypointsImpedanceController
            , virtual public mp::MPPool
//            , virtual public NJointKeypointsImpedanceMPControllerInterface
    {
    public:
        NJointKeypointsImpedanceMPController(const RobotUnitPtr& robotUnit, const NJointControllerConfigPtr& config, const VirtualRobot::RobotPtr&);

        std::string getClassName(const Ice::Current&) const override;
        void reconfigureController(const std::string &filename, const Ice::Current &) override;
        void rtPreActivateController() override;
        void additionalTask() override;



        //        // NJointControllerInterface interface
        //    public:
        //        void deactivateController(const Ice::Current &) override;
    };
}
